/*
 * nvs.cpp
 *
 *  Created on: 07.07.2017
 *      Author: Wolle
 */
#include "nvs_ext.h"



NVS::NVS(const char* Name) {
	nvsname = Name;
	nvserr = 0;
}

NVS::~NVS() {
	close();
}

void NVS::open() {
	 if (!nvshandle) {                                        		// Opened already?
		nvserr = nvs_open(nvsname, NVS_READWRITE, &nvshandle);   	// No, open nvs
		if (nvserr) {
			sprintf(sbuf, "nvsopen \"%s\" failed! %s\n", nvsname, error(nvserr).c_str());
			if(nvs_info) nvs_info(sbuf);
		} else {
			sprintf(sbuf, "nvsopen \"%s\" successful\n", nvsname);
			if(nvs_info) nvs_info(sbuf);
		}
	}
}

void NVS::close() {
	if (nvshandle) {
		nvs_close(nvshandle);
		sprintf(sbuf, "nvs \"%s\" closed\n", nvsname);
		if(nvs_info) nvs_info(sbuf);
	}
}

esp_err_t NVS::clear() {
	open();                                         	// Be sure to open nvs
	nvserr = nvs_erase_all(nvshandle);                	// Clear all keys
	if (nvserr) {
		sprintf(sbuf, "nvsclear \"%s\"failed %s\n", nvsname, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
	} else {
		sprintf(sbuf, "nvsclear \"%s\"successful\n", nvsname);
		if(nvs_info) nvs_info(sbuf);
	}
	return nvserr;
}

String NVS::getstr(const char* key) {
	char nvs_buf[NVSBUFSIZE];       		// Buffer for contents
	size_t len = NVSBUFSIZE;      			// Max length of the string, later real length
	nvs_buf[0] = '\0';						// Return empty string on error
	nvserr = nvs_get_str(nvshandle, key, nvs_buf, &len);
	if (nvserr) {
		sprintf(sbuf, "nvsgetstr failed for key %s %s\n", key, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
		return "";
	} else {
		sprintf(sbuf, "nvsgetstr %s=%s\n", key, nvs_buf);
		if(nvs_info) nvs_info(sbuf);
	}
	return String(nvs_buf);
}

esp_err_t NVS::setstr(const char* key, String val) {
	char nvs_buf[NVSBUFSIZE];       		// Buffer for contents
	size_t len = NVSBUFSIZE;      			// Max length of the string, later real length
	nvs_buf[0] = '\0';						// Return empty string on error

	String curcont;												// Current contents
	bool wflag = false;											// Assume update or new key

	if (val.length() >= NVSBUFSIZE) {							// Limit length of string to store
		sprintf(sbuf, "nvsetstr %s length failed!\n", key);
		if(nvs_info) nvs_info(sbuf);
		return ESP_ERR_NVS_NOT_ENOUGH_SPACE;
	}
	if (searchstr(key)) {										// Already in nvs?
		nvserr = nvs_get_str(nvshandle, key, nvs_buf, &len);
		curcont =nvs_buf;							// Read current value
		wflag = (curcont.c_str() != val.c_str());				// Value change?
	}
	nvserr = nvs_set_str(nvshandle, key, val.c_str()); 			// Store key and value
	if (nvserr) {												// Check error
		sprintf(sbuf, "nvssetstr failed for key %s %s\n", key, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
	} else {
		if (wflag) {											// Update or new?
			sprintf(sbuf, "nvssetstr %s=%s updated\n", key, val.c_str());
			if(nvs_info) nvs_info(sbuf);
		} else {
			sprintf(sbuf, "nvssetstr %s=%s \n", key, val.c_str());
			if(nvs_info) nvs_info(sbuf);
		}
	}
	return nvserr;
}

esp_err_t NVS::setint(const char* key, int32_t val) {
	int32_t curcont;											// Current contents
	bool wflag = false;											// Assume update or new key
	if (nvs_get_i32(nvshandle, key, &curcont) == ESP_OK) {    	// Already in nvs?
		wflag = (curcont != val);								// Value change?
	}
	nvserr = nvs_set_i32(nvshandle, key, val); // Store key and value
	if (nvserr) {
		sprintf(sbuf, "nvssetint failed for key %s %s\n", key, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
	} else {
		if (wflag) {											// Update or new?
			sprintf(sbuf, "nvssetint %s=%i updated\n", key, val);
			if(nvs_info) nvs_info(sbuf);
		} else {
			sprintf(sbuf, "nvssetint %s=%i \n", key, val);
			if(nvs_info) nvs_info(sbuf);
		}
	}
	return nvserr;
}
int32_t NVS::getint(const char* key) {
	int32_t val;
	nvserr = nvs_get_i32(nvshandle, key, &val);					// Store key and value
	if (nvserr > 0) {
		sprintf(sbuf, "nvsgetint failed for key %s %s\n", key, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
	} else {
		sprintf(sbuf, "nvsgetint %s=%i\n", key, val);
		if(nvs_info) nvs_info(sbuf);
	}
	return val;
}

esp_err_t NVS::setlonglong(const char* key, int64_t val) {
	int64_t curcont;											// Current contents
	bool wflag = false;											// Assume update or new key

	if (nvs_get_i64(nvshandle, key, &curcont) == ESP_OK) {		// Already in nvs?
		wflag = (curcont != val);                           	// Value change?
	}
	nvserr = nvs_set_i64(nvshandle, key, val);					// Store key and value
	if (nvserr) {
		sprintf(sbuf, "nvssetlonglong failed for key %s %s\n" ,key, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
	} else {
		if (wflag) {											// Update or new?
			sprintf(sbuf, "nvssetlonglong %s=%lli updated\n", key, val);
			if(nvs_info) nvs_info(sbuf);
		} else {
			sprintf(sbuf, "nvssetlonglong %s=%lli \n", key, val);
			if(nvs_info) nvs_info(sbuf);
		}
	}
	return nvserr;
}

int64_t NVS::getlonglong(const char* key) {
	int64_t val;
	nvserr = nvs_get_i64(nvshandle, key, &val);					// Store key and value
	if (nvserr) {
		sprintf(sbuf, "nvsgetlonglong failed for key %s %s\n", key, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
	} else {
		sprintf(sbuf, "nvsgetlonglong %s=%lli\n", key, val);
		if(nvs_info) nvs_info(sbuf);
	}
	return val;
}

esp_err_t NVS::setfloat(const char* key, const float_t val) {
	size_t fl=sizeof(float);
	bool wflag = false;
	float_t curcont=0;
	if (nvs_get_blob(nvshandle, key, (void*) &curcont, &fl) == ESP_OK) {		// Already in nvs?
			wflag = (curcont != val);                           	// Value change?
		}
	nvserr = nvs_set_blob(nvshandle, key, (void*) &val ,sizeof(float_t)); 			// Store key and value
	if (nvserr) {												// Check error
		sprintf(sbuf, "nvssetfloat failed for key %s %s\n", key, error(nvserr).c_str());
		if(nvs_info) nvs_info(sbuf);
	} else {
		if (wflag) {											// Update or new?
			sprintf(sbuf, "nvssetfloat %s=%f updated\n", key, val);
			if(nvs_info) nvs_info(sbuf);
	} else {
			sprintf(sbuf, "nvssetfloat %s=%f \n", key, val);
			if(nvs_info) nvs_info(sbuf);
			}
	}

	return nvserr;
}

float_t NVS::getfloat(const char* key) {
	float_t value=0;
	size_t bloblen=sizeof(float_t);
	nvserr = nvs_get_blob(nvshandle, key, (void*) &value ,&bloblen);
	if (nvserr) {
			sprintf(sbuf, "nvsgetfloat failed for key %s %s\n", key, error(nvserr).c_str());
			if(nvs_info) nvs_info(sbuf);
		} else {
			sprintf(sbuf, "nvsgetfloat %s=%f\n", key, value);
			if(nvs_info) nvs_info(sbuf);
		}
		return value;
}


bool NVS::searchstr(const char* key) {
	size_t len = NVSBUFSIZE;									// Length of the string

	nvserr = nvs_get_str(nvshandle, key, NULL, &len);			// Get length of contents
	return (nvserr == ESP_OK);									// Return true if found
}

String NVS::error(esp_err_t nvserr){
	String err;
	switch(nvserr){
	case 0x1100: err="ESP_ERR_NVS_BASE"; break;
	case 0x1101: err="ESP_ERR_NVS_NOT_INITIALIZED"; break;
	case 0x1102: err="ESP_ERR_NVS_NOT_FOUND"; break;
	case 0x1103: err="ESP_ERR_NVS_TYPE_MISMATCH"; break;
	case 0x1104: err="ESP_ERR_NVS_READ_ONLY"; break;
	case 0x1105: err="ESP_ERR_NVS_NOT_ENOUGH_SPACE"; break;
	case 0x1106: err="ESP_ERR_NVS_INVALID_NAME"; break;
	case 0x1107: err="ESP_ERR_NVS_INVALID_HANDLE"; break;
	case 0x1108: err="ESP_ERR_NVS_REMOVE_FAILED"; break;
	case 0x1109: err="ESP_ERR_NVS_KEY_TOO_LONG"; break;
	case 0x110A: err="ESP_ERR_NVS_PAGE_FULL"; break;
	case 0x110B: err="ESP_ERR_NVS_INVALID_STATE"; break;
	case 0x110C: err="ESP_ERR_NVS_INVALID_LENGTH"; break;
	case 0x110D: err="ESP_ERR_NVS_NO_FREE_PAGES"; break;
	default: 	 err="ESP_ERR_UNKNOWN"; break;
	}
	return err;
}
